--------------------------------------------------------------------------------
-- Company: 		Trenz Electronic GmbH
-- Engineer: 		Oleksandr Kiyenko
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.all;
--------------------------------------------------------------------------------
entity iic_pca9548a is
generic (
	C_IIC_ADDR				: STD_LOGIC_VECTOR( 7 downto 0)	:= x"70"
);
port (
	clk_in					: in  STD_LOGIC;
  rst_n           : in  STD_LOGIC;
	-- I2C - Slave Interface
	s_scl_i					: out STD_LOGIC;
	s_scl_o					: in  STD_LOGIC;
	s_scl_t					: in  STD_LOGIC;
	s_sda_i					: out STD_LOGIC;
	s_sda_o					: in  STD_LOGIC;
	s_sda_t					: in  STD_LOGIC;
  -- we need a master port to act as slave or we just route a 4-wire interface
  
	-- I2C - Master Interface 0
	m00_scl_i				: in  STD_LOGIC;
	m00_scl_o				: out STD_LOGIC;
	m00_scl_t				: out STD_LOGIC;
	m00_sda_i				: in  STD_LOGIC;
	m00_sda_o				: out STD_LOGIC;
	m00_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 1
	m01_scl_i				: in  STD_LOGIC;
	m01_scl_o				: out STD_LOGIC;
	m01_scl_t				: out STD_LOGIC;
	m01_sda_i				: in  STD_LOGIC;
	m01_sda_o				: out STD_LOGIC;
	m01_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 2
	m02_scl_i				: in  STD_LOGIC;
	m02_scl_o				: out STD_LOGIC;
	m02_scl_t				: out STD_LOGIC;
	m02_sda_i				: in  STD_LOGIC;
	m02_sda_o				: out STD_LOGIC;
	m02_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 3
	m03_scl_i				: in  STD_LOGIC;
	m03_scl_o				: out STD_LOGIC;
	m03_scl_t				: out STD_LOGIC;
	m03_sda_i				: in  STD_LOGIC;
	m03_sda_o				: out STD_LOGIC;
	m03_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 4
	m04_scl_i				: in  STD_LOGIC;
	m04_scl_o				: out STD_LOGIC;
	m04_scl_t				: out STD_LOGIC;
	m04_sda_i				: in  STD_LOGIC;
	m04_sda_o				: out STD_LOGIC;
	m04_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 5
	m05_scl_i				: in  STD_LOGIC;
	m05_scl_o				: out STD_LOGIC;
	m05_scl_t				: out STD_LOGIC;
	m05_sda_i				: in  STD_LOGIC;
	m05_sda_o				: out STD_LOGIC;
	m05_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 6
	m06_scl_i				: in  STD_LOGIC;
	m06_scl_o				: out STD_LOGIC;
	m06_scl_t				: out STD_LOGIC;
	m06_sda_i				: in  STD_LOGIC;
	m06_sda_o				: out STD_LOGIC;
	m06_sda_t				: out STD_LOGIC;
	-- I2C - Master Interface 7
	m07_scl_i				: in  STD_LOGIC;
	m07_scl_o				: out STD_LOGIC;
	m07_scl_t				: out STD_LOGIC;
	m07_sda_i				: in  STD_LOGIC;
	m07_sda_o				: out STD_LOGIC;
	m07_sda_t				: out STD_LOGIC
);
end iic_pca9548a;
--------------------------------------------------------------------------------
architecture arch_imp of iic_pca9548a is
--------------------------------------------------------------------------------
signal scl_sr		: STD_LOGIC_VECTOR(1 downto 0)	:= "00";
signal sda_sr		: STD_LOGIC_VECTOR(1 downto 0)	:= "00";
signal scl_rising	: BOOLEAN 						:= FALSE;
signal scl_falling	: BOOLEAN 						:= FALSE;
signal i2c_start	: BOOLEAN 						:= FALSE;
signal i2c_stop		: BOOLEAN 						:= FALSE;
--------------------------------------------------------------------------------
signal cycle_cnt	: integer range 0 to 10			:= 0;
signal data_in_sr	: STD_LOGIC_VECTOR(7 downto 0)	:= x"00";
signal data_out_sr	: STD_LOGIC_VECTOR(7 downto 0)	:= x"00";
signal dev_sel		: STD_LOGIC 					:= '0';
signal rnw			: STD_LOGIC 					:= '0';
signal ch_sel_reg	: STD_LOGIC_VECTOR(7 downto 0)	:= x"00";
signal sda_int_drv	: STD_LOGIC 					:= '1';
signal sda_ext_drv	: STD_LOGIC 					:= '1';
signal scl_ext_drv	: STD_LOGIC 					:= '1';
type sm_state_type is (ST_IDLE, ST_DEV_SEL, ST_DATA_WR, ST_DATA_RD);
signal sm_state		: sm_state_type 				:= ST_IDLE;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
process(clk_in)
begin
	if(clk_in = '1' and clk_in'event)then
		scl_sr		<= scl_sr(0) & (s_scl_o or s_scl_t);
		sda_sr		<= sda_sr(0) & (s_sda_o or s_sda_t);
	end if;
end process;
scl_rising			<= (scl_sr = "01");
scl_falling			<= (scl_sr = "10");
i2c_start			<= (sda_sr = "10") and (scl_sr = "11");
i2c_stop			<= (sda_sr = "01") and (scl_sr = "11");
--------------------------------------------------------------------------------
process(clk_in)
begin
	if(clk_in = '1' and clk_in'event)then
    if rst_n = '0' then
      sm_state <= ST_IDLE;
    else
      case sm_state is
      
        when ST_IDLE	=>
          cycle_cnt					<= 0;
          sda_int_drv					<= '1';
          dev_sel						<= '0';
          if(i2c_start)then			-- Start condition
            sm_state				<= ST_DEV_SEL;
          end if;
          
        when ST_DEV_SEL	=>
          if(scl_rising)then
            if(cycle_cnt = 7)then
              if(data_in_sr(6 downto 0) = C_IIC_ADDR(6 downto 0))then
                dev_sel			<= '1';
                rnw				<= sda_sr(0);
              else
                dev_sel			<= '0';
              end if;
            end if;
            data_in_sr				<= data_in_sr(6 downto 0) & sda_sr(0);
            cycle_cnt				<= cycle_cnt + 1;
          end if;
          if(scl_falling)then
            if(cycle_cnt = 8)then
              cycle_cnt			<= 0;			-- End of cycle
              if(dev_sel = '1')then
                sda_int_drv		<= '0';			-- Drive SDA (ACK)
                if(rnw = '0')then				-- Write access
                  sm_state	<= ST_DATA_WR;
                else							-- Read access
                  data_out_sr	<= ch_sel_reg;	-- Data to be sent
                  sm_state	<= ST_DATA_RD;
                end if;
              else
                sda_int_drv		<= '1';
                sm_state		<= ST_IDLE;		-- Wait for next start
              end if;
            end if;
          end if;
          
        when ST_DATA_WR =>	-- ######## Write register stage ########
          if(i2c_stop)then
            sm_state				<= ST_IDLE;
          end if;
          if(scl_rising)then
            data_in_sr				<= data_in_sr(6 downto 0) & sda_sr(0);
          end if;
          if(scl_falling)then
            if(cycle_cnt = 8)then
              sda_int_drv			<= '0';			-- Drive SDA for ACK
              ch_sel_reg			<= data_in_sr;
            elsif(cycle_cnt = 9)then
              sda_int_drv			<= '1';			-- Free SDA after ACK
              --cycle_cnt			<= 0;			-- End of cycle
              sm_state			<= ST_IDLE;
            else
              sda_int_drv			<= '1';			-- Free SDA after ACK
            end if;
            cycle_cnt				<= cycle_cnt + 1;
          end if;
          
        when ST_DATA_RD =>			-- ######## Read register stage ########
          if(i2c_stop)then
            sm_state				<= ST_IDLE;
          end if;
          if(scl_falling)then
            data_out_sr				<= data_out_sr(6 downto 0) & '1';	-- Shift
            if(cycle_cnt = 8)then
              sda_int_drv			<= '1';			-- Free bus to see NACK
            elsif(cycle_cnt = 9)then
              sda_int_drv			<= '0';			-- Drive SDA to send Stop
            else
              sda_int_drv			<= data_out_sr(7);	-- Drive SDA for data output
            end if;
            cycle_cnt				<= cycle_cnt + 1;
          end if;
          if(scl_rising)then
            if(cycle_cnt = 10)then
              sda_int_drv			<= '1';			-- Drive SDA to send Stop
              sm_state			<= ST_IDLE;
            end if;
          end if;
      end case;
    end if;
	end if;
end process;
--------------------------------------------------------------------------------
process(clk_in)
begin
	if(clk_in = '1' and clk_in'event)then
		case ch_sel_reg is
			when x"01"	=> s_scl_i <= m00_scl_i and s_scl_t;
			when x"02"	=> s_scl_i <= m01_scl_i and s_scl_t;
			when x"04"	=> s_scl_i <= m02_scl_i and s_scl_t;
			when x"08"	=> s_scl_i <= m03_scl_i and s_scl_t;
			when x"10"	=> s_scl_i <= m04_scl_i and s_scl_t;
			when x"20"	=> s_scl_i <= m05_scl_i and s_scl_t;
			when x"40"	=> s_scl_i <= m06_scl_i and s_scl_t;
			when x"80"	=> s_scl_i <= m07_scl_i and s_scl_t;
			when others => s_scl_i <= s_scl_t;
		end case;
	end if;
end process;

process(clk_in)
begin
	if(clk_in = '1' and clk_in'event)then
		case ch_sel_reg is
			when x"01"	=> s_sda_i <= m00_sda_i and sda_int_drv and s_sda_t;
			when x"02"	=> s_sda_i <= m01_sda_i and sda_int_drv and s_sda_t;
			when x"04"	=> s_sda_i <= m02_sda_i and sda_int_drv and s_sda_t;
			when x"08"	=> s_sda_i <= m03_sda_i and sda_int_drv and s_sda_t;
			when x"10"	=> s_sda_i <= m04_sda_i and sda_int_drv and s_sda_t;
			when x"20"	=> s_sda_i <= m05_sda_i and sda_int_drv and s_sda_t;
			when x"40"	=> s_sda_i <= m06_sda_i and sda_int_drv and s_sda_t;
			when x"80"	=> s_sda_i <= m07_sda_i and sda_int_drv and s_sda_t;
			when others => s_sda_i <= sda_int_drv and s_sda_t;
		end case;
	end if;
end process;

process(clk_in)
begin
	if(clk_in = '1' and clk_in'event)then
		m00_scl_t	<= s_scl_t or (not ch_sel_reg(0));
		m01_scl_t	<= s_scl_t or (not ch_sel_reg(1));
		m02_scl_t	<= s_scl_t or (not ch_sel_reg(2));
		m03_scl_t	<= s_scl_t or (not ch_sel_reg(3));
		m04_scl_t	<= s_scl_t or (not ch_sel_reg(4));
		m05_scl_t	<= s_scl_t or (not ch_sel_reg(5));
		m06_scl_t	<= s_scl_t or (not ch_sel_reg(6));
		m07_scl_t	<= s_scl_t or (not ch_sel_reg(7));

		m00_sda_t	<= s_sda_t or (not ch_sel_reg(0));
		m01_sda_t	<= s_sda_t or (not ch_sel_reg(1));
		m02_sda_t	<= s_sda_t or (not ch_sel_reg(2));
		m03_sda_t	<= s_sda_t or (not ch_sel_reg(3));
		m04_sda_t	<= s_sda_t or (not ch_sel_reg(4));
		m05_sda_t	<= s_sda_t or (not ch_sel_reg(5));
		m06_sda_t	<= s_sda_t or (not ch_sel_reg(6));
		m07_sda_t	<= s_sda_t or (not ch_sel_reg(7));
	end if;
end process;

m00_scl_o	<= '0';
m01_scl_o	<= '0';
m02_scl_o	<= '0';
m03_scl_o	<= '0';
m04_scl_o	<= '0';
m05_scl_o	<= '0';
m06_scl_o	<= '0';
m07_scl_o	<= '0';

m00_sda_o	<= '0';
m01_sda_o	<= '0';
m02_sda_o	<= '0';
m03_sda_o	<= '0';
m04_sda_o	<= '0';
m05_sda_o	<= '0';
m06_sda_o	<= '0';
m07_sda_o	<= '0';
--------------------------------------------------------------------------------
end arch_imp;
